use strict;
use POSIX;
use Net::SMTP;

my $systemname = 'attila.bofh.it';
my $maintainermail = 'usenet (at) bofh.it';
my $nulladdr = 'dev.null@attila.bofh.it';
my $maxlines = 20;
my $logfile = '/var/log/news/ar.log';
my $testgroups = '\.test$';
my $ignorewords = 'ignore';

##############################################################################
my $debug = 0;
my @template;

my %seen_addr;
my $last_purge = time;

# number of seconds to remember an address (since the last one seen)
my $remember_time = 60 * 60 * 6;
# maximum number of messages sent in the $t1 window
my $msg_max = 4;
my $t1 = 60 * 60 * 6;

sub check_rate_limit ($) {
	my ($addr) = @_;

	my $now = time;
	if ($last_purge > $now - 60 * 30) {
		foreach (keys %seen_addr) {
			my ($seen, $num) = split(/ /, $seen_addr{$_});
			delete $seen_addr{$addr} if $seen < $now - $remember_time;
		}
	}

	if (not $seen_addr{$addr}) {
		$seen_addr{$addr} = "$now 1";
		return 0;	# OK
	}
	
	my ($seen, $num) = split(/ /, $seen_addr{$addr});
	$seen_addr{$addr} = $seen . ' ' . ($num + 1);

	return 1 if $seen > $now - $t1 and $num >= $msg_max;	# TOO MANY
	return 0;		# OK
}

# strip comments from a RFC 822 address
sub parseaddr ($) {
	local ($_) = @_;
	my $hdr = (/<(\S+)>/)[0];			# XXX should use Mail::Address
	$hdr = (split(' '))[0] unless $hdr;
	$hdr;
}

# check for obviously invalid addresses
sub invalidaddr ($) {
	local ($_) = @_;
	return 1 unless /\@[a-zA-Z\d\.-]+\.(?:\w{2}|com|net|org|edu|mil|gov|int|info|biz|name|pro|aero|coop|museum)$/;
	return 1 if /(?:togli|xxx|spam|remove|news@|news\.tin\.it|http:|\.\.|\@.*\@|\@-|\@\.)/i;
	return 0;
}

sub sendmail {
	my ($from, $to, $data) = @_;

	if ($debug > 1) {
		print STDERR map { "$_\n" } (@$data);
		return 1 if $debug > 2;
	}

	my $conn = Net::SMTP->new('localhost', Hello => 'autoresponder');
	if (not $conn) {
		print STDERR "Cannot connect to mail server\n";
		return 0;
	}
	if (not $conn->mail($from)) {
		print STDERR "FROM: ${conn}->message\n";
		return 0;
	}
	if (not $conn->to($to)) {
		print STDERR "TO: ${conn}->message (TO: $to)\n";
		return 0;
	}
	if (not $conn->data(map { "$_\n" } (@$data))) {
		print STDERR "DATA: ${conn}->message (TO: $to)\n";
		return 0;
	}
	$conn->quit;
	return 1;
}

##############################################################################
sub autoresponder_init {
	@template = <DATA>;
	chop @template;

	if ($debug > 0) {
		open(LOG, '>&STDERR');
	} elsif ($logfile) {
		open(LOG, ">>$logfile") or die "Can't open log: $!";
		my $oldfh = select LOG; $| = 1; select $oldfh;
		print LOG "start\n";
	}
}

sub autoresponder_main {
	my ($token, $hdr, $headers, $body) = @_;

	# skip really broken articles
	return if not $hdr->{subject} and $hdr->{newsgroups};
	return if $hdr->{control};

	# skip non-test groups
	my @ng = split(/,/, $hdr->{newsgroups});
	return if not grep(/$testgroups/o, @ng);

	return if $hdr->{subject} =~ /$ignorewords/o;

	$$body =~ /((?:.*?\n){0,$maxlines})/mo;
	my $bodystart = $1;
	return if $bodystart =~ /$ignorewords/o;

	my $replyto = parseaddr($hdr->{'reply-to'} || $hdr->{from} || return);

	return if invalidaddr($replyto);

	if (check_rate_limit($replyto)) {
		print LOG "SKIP $replyto ($hdr->{from}) [$hdr->{subject}]\n";
		return;
	}

	my $art = $$headers . "\n" . $bodystart;
	$art =~ s/^/| /mg;

	my @path = reverse split(/[^\w\._-]/, $hdr->{path});
	shift @path;

	my %subst = (
		localdate => POSIX::strftime("%a %b %d %Y %T %z (%Z)", localtime time),
		subject			=> "Answer from autoresponder ($hdr->{subject})",
		date			=> $hdr->{date},
		path			=> join("\n\t", @path),
		newsgroups		=> join("\n\t", split(/,/, $hdr->{newsgroups})),
		article			=> $art,
		replyto			=> $replyto,
		fromaddr		=> $nulladdr,
		lines			=> $maxlines,
		systemname		=> $systemname,
		maintainermail	=> $maintainermail,
	);

	my @temp = @template;
	foreach (@temp) {
		if (/%%(\w+)%%/) {
			die "unknown %%word%%: $1\n" if not exists $subst{$1};
			s/%%(\w+)%%/eval { $subst{$1} }/eg;
		}
	}

	print LOG "REPLY $replyto ($hdr->{from}) [$hdr->{subject}]\n";
	sendmail($nulladdr, $replyto, \@temp);
}

1;

##############################################################################
__DATA__
From: %%fromaddr%% (%%systemname%% autoresponder)
To: %%replyto%%
Subject: %%subject%%
Precedence: bulk

[TESTO ITALIANO IN FONDO AL MESSAGGIO]

Hello,

this message is never sent unsolicited. Don't panic.

You (or more precisely someone using your mail address
%%replyto%%) have posted a test article to
the following newsgroups:

	%%newsgroups%%

It was sent on %%date%% and travelled along this
way from your site to %%systemname%%:

	%%path%%

where it arrived at %%localdate%%

Automatic replies are sent to anybody posting into a group ending with
the word ".test". If you don't want to receive these messages, include
the words 'ignore' at the top of your articles or in the Subject header.
Anyway, no more than 4 replies every 6 hours are sent.

For obvious reasons, mails to %%fromaddr%% are directed
into /dev/null. If you feel to write us, send your mail to
%%maintainermail%% instead.

Follows the full headers plus the first %%lines%% lines of your article.

-----------------------------------------------------------------------

Ciao,

questo messaggio non viene mai inviato non richiesto. Niente paura.

Lo hai ricevuto perche` tu, o piu` precisamente qualcuno che ha usato
il tuo indirizzo di email %%replyto%%, hai
inviato un articolo di prova nei seguenti newsgroup:

	%%newsgroups%%

Era stato mandato il giorno %%date%% e ha seguito
questo percorso per arrivare dal tuo sito a %%systemname%%:

	%%path%%

dove e` arrivato il giorno %%localdate%%

Le risposte automatiche sono inviate a chiunque invii un articolo a un
newsgroup che termina con ".test". Se non vuoi piu` riceverle aggiungi
la parola "ignore" all'inizio di tuoi articoli o nell'header Subject.
In ogni caso, non vengono inviate piu` di 4 risposte ogni 6 ore.

Per ovvie ragioni la posta inviata a %%fromaddr%% finisce
in /dev/null. Se hai bisogno di scriverci puoi farlo all'indirizzo
%%maintainermail%%.

Seguono gli header completi e le prime %%lines%% linee del tuo articolo.

%%article%%
-- 
autoresponder 0.1 by Marco d'Itri - http://www.linux.it/~md/software/
